@@ -4,15 +4,18 @@ import logging
 
 from collections import defaultdict
 from collections.abc import Mapping
+from itertools import chain
 from pathlib import Path
 from typing import TYPE_CHECKING
 from typing import Any
 from typing import Literal
-from typing import Union
-from typing import cast
 
+from packaging.licenses import InvalidLicenseExpression
+from packaging.licenses import canonicalize_license_expression
 from packaging.utils import canonicalize_name
 
+from poetry.core.packages.dependency import Dependency
+from poetry.core.packages.dependency_group import DependencyGroup
 from poetry.core.utils.helpers import combine_unicode
 from poetry.core.utils.helpers import readme_content_type
 
@@ -20,17 +23,12 @@ from poetry.core.utils.helpers import readme_content_type
 if TYPE_CHECKING:
     from packaging.utils import NormalizedName
 
-    from poetry.core.packages.dependency import Dependency
-    from poetry.core.packages.dependency_group import DependencyGroup
     from poetry.core.packages.project_package import ProjectPackage
     from poetry.core.poetry import Poetry
     from poetry.core.pyproject.toml import PyProjectTOML
-    from poetry.core.spdx.license import License
 
-    DependencyConstraint = Union[str, Mapping[str, Any]]
-    DependencyConfig = Mapping[
-        str, Union[list[DependencyConstraint], DependencyConstraint]
-    ]
+    DependencyConstraint = str | Mapping[str, Any]
+    DependencyConfig = Mapping[str, list[DependencyConstraint] | DependencyConstraint]
 
 
 logger = logging.getLogger(__name__)
@@ -87,7 +85,27 @@ class Factory:
         return ProjectPackage(name, version)
 
     @classmethod
-    def _add_package_group_dependencies(
+    def _add_package_pep735_group_dependencies(
+        cls,
+        package: ProjectPackage,
+        group: DependencyGroup,
+        dependencies: list[str | dict[str, str]],
+    ) -> list[str]:
+        group_includes = []
+        for constraint in dependencies:
+            if isinstance(constraint, str):
+                dep = Dependency.create_from_pep_508(
+                    constraint,
+                    relative_to=package.root_dir,
+                    groups=[group.pretty_name],
+                )
+                group.add_dependency(dep)
+            elif include := constraint.get("include-group"):
+                group_includes.append(include)
+        return group_includes
+
+    @classmethod
+    def _add_package_poetry_group_dependencies(
         cls,
         package: ProjectPackage,
         group: str | DependencyGroup,
@@ -134,13 +152,18 @@ class Factory:
     ) -> None:
         project = pyproject.data.get("project", {})
         tool_poetry = pyproject.poetry_config
+        dependency_groups = pyproject.data.get("dependency-groups", {})
 
         package.root_dir = root
 
         cls._configure_package_metadata(package, project, tool_poetry, root)
         cls._configure_entry_points(package, project, tool_poetry)
         cls._configure_package_dependencies(
-            package, project, tool_poetry, with_groups=with_groups
+            package=package,
+            project=project,
+            tool_poetry=tool_poetry,
+            dependency_groups=dependency_groups,
+            with_groups=with_groups,
         )
         cls._configure_package_poetry_specifics(package, tool_poetry)
 
@@ -177,14 +200,46 @@ class Factory:
         package.description = project.get("description") or tool_poetry.get(
             "description", ""
         )
+        raw_license: str | None = None
         if project_license := project.get("license"):
             if isinstance(project_license, str):
-                raw_license = project_license
+                try:
+                    package.license_expression = canonicalize_license_expression(
+                        project_license
+                    )
+                except InvalidLicenseExpression:
+                    # This is handled in validate().
+                    raw_license = project_license
             else:
-                raw_license = project_license.get("text", "")
-                if not raw_license and (
-                    license_file := cast(str, project_license.get("file", ""))
-                ):
+                # Table values for the license key in the [project] table,
+                # including the text and file table subkeys, are now deprecated.
+                # If the new license-files key is present, build tools MUST raise an
+                # error if the license key is defined and has a value other
+                # than a single top-level string.
+                # https://peps.python.org/pep-0639/#deprecate-license-key-table-subkeys
+                if "license-files" in project:
+                    raise ValueError(
+                        "[project.license] must be of type string"
+                        " if [project.license-files] is defined."
+                    )
+
+                # Tools MUST NOT use the contents of the license.text [project] key
+                # (or equivalent tool-specific format), [...] to fill [...] the Core
+                # Metadata License-Expression field without informing the user and
+                # requiring unambiguous, affirmative user action to select and confirm
+                # the desired license expression value before proceeding.
+                # https://peps.python.org/pep-0639/#converting-legacy-metadata
+                # -> We just set the old license field in this case
+                #    (and give a warning in validate).
+                raw_license = project_license.get("text")
+                if not raw_license and (license_file := project_license.get("file")):
+                    # If the specified license file is present in the source tree,
+                    # build tools SHOULD use it to fill the License-File field
+                    # in the core metadata, and MUST include the specified file
+                    # as if it were specified in a license-file field.
+                    # If the file does not exist at the specified path,
+                    # tools MUST raise an informative error as previously specified.
+                    # https://peps.python.org/pep-0639/#deprecate-license-key-table-subkeys
                     license_path = (root / license_file).absolute()
                     try:
                         raw_license = Path(license_path).read_text(encoding="utf-8")
@@ -192,13 +247,36 @@ class Factory:
                         raise FileNotFoundError(
                             f"Poetry: license file '{license_path}' not found"
                         ) from e
+                    else:
+                        # explicitly not a tuple to allow default handling
+                        # to find additional license files later
+                        package.license_files = Path(license_file)
         else:
-            raw_license = tool_poetry.get("license", "")
-        try:
-            license_: License | None = license_by_id(raw_license)
-        except ValueError:
-            license_ = None
-        package.license = license_
+            raw_license = tool_poetry.get("license")
+        if raw_license:
+            package.license = license_by_id(raw_license)
+
+        # important: distinction between empty array and None:
+        # - empty array: explicitly no license files
+        # - None (not set): default handling allowed
+        if (license_files := project.get("license-files")) is not None:
+            # Build tools MUST treat each value as a glob pattern,
+            # and MUST raise an error if the pattern contains invalid glob syntax.
+            # https://peps.python.org/pep-0639/#add-license-files-key
+            for entry in license_files:
+                if "\\" in entry:
+                    # Path delimiters MUST be the forward slash character (/).
+                    raise ValueError(
+                        f"Invalid entry in [project.license-files]: '{entry}'"
+                        " (Path delimiters must be forward slashes.)"
+                    )
+                if ".." in Path(entry).parts:
+                    # Parent directory indicators (..) MUST NOT be used.
+                    raise ValueError(
+                        f"Invalid entry in [project.license-files]: '{entry}'"
+                        " ('..' must not be used.)"
+                    )
+            package.license_files = tuple(license_files)
 
         package.requires_python = project.get("requires-python", "*")
         package.keywords = project.get("keywords") or tool_poetry.get("keywords", [])
@@ -289,18 +367,21 @@ class Factory:
         package: ProjectPackage,
         project: dict[str, Any],
         tool_poetry: dict[str, Any],
+        dependency_groups: dict[str, list[str | dict[str, str]]],
         with_groups: bool = True,
     ) -> None:
         from poetry.core.packages.dependency import Dependency
         from poetry.core.packages.dependency_group import MAIN_GROUP
         from poetry.core.packages.dependency_group import DependencyGroup
 
-        dependencies = project.get("dependencies", {})
+        dependencies = project.get("dependencies")
         optional_dependencies = project.get("optional-dependencies", {})
         dynamic = project.get("dynamic", [])
 
         package_extras: dict[NormalizedName, list[Dependency]]
-        if dependencies or optional_dependencies:
+        if dependencies is not None or optional_dependencies:
+            dependencies = dependencies or {}
+
             group = DependencyGroup(
                 MAIN_GROUP,
                 mixed_dynamic=(
@@ -333,25 +414,19 @@ class Factory:
             package.extras = package_extras
 
         if "dependencies" in tool_poetry:
-            cls._add_package_group_dependencies(
+            cls._add_package_poetry_group_dependencies(
                 package=package,
                 group=MAIN_GROUP,
                 dependencies=tool_poetry["dependencies"],
             )
 
-        if with_groups and "group" in tool_poetry:
-            for group_name, group_config in tool_poetry["group"].items():
-                group = DependencyGroup(
-                    group_name, optional=group_config.get("optional", False)
-                )
-                cls._add_package_group_dependencies(
-                    package=package,
-                    group=group,
-                    dependencies=group_config["dependencies"],
-                )
+        if with_groups:
+            cls._configure_package_dependency_groups(
+                package, tool_poetry, dependency_groups
+            )
 
         if with_groups and "dev-dependencies" in tool_poetry:
-            cls._add_package_group_dependencies(
+            cls._add_package_poetry_group_dependencies(
                 package=package,
                 group="dev",
                 dependencies=tool_poetry["dev-dependencies"],
@@ -377,6 +452,74 @@ class Factory:
 
             package.extras = package_extras
 
+    @classmethod
+    def _configure_package_dependency_groups(
+        cls,
+        package: ProjectPackage,
+        tool_poetry: dict[str, Any],
+        dependency_groups: dict[str, list[str | dict[str, str]]],
+    ) -> None:
+        tool_poetry_groups = tool_poetry.get("group", {})
+        tool_poetry_groups_normalized = {
+            canonicalize_name(name): config
+            for name, config in tool_poetry_groups.items()
+        }
+        # create groups from the dependency-groups section considering
+        # additional information from the corresponding tool.poetry.group section
+        pep739_include_groups = {}
+        for group_name, dependencies in dependency_groups.items():
+            poetry_group_config = tool_poetry_groups_normalized.get(
+                canonicalize_name(group_name), {}
+            )
+            group = DependencyGroup(
+                name=group_name,
+                optional=poetry_group_config.get("optional", False),
+            )
+            package.add_dependency_group(group)
+            included_groups = cls._add_package_pep735_group_dependencies(
+                package=package,
+                group=group,
+                dependencies=dependencies,
+            )
+            pep739_include_groups[group_name] = included_groups
+        # create groups from the tool.poetry.group section
+        # with no corresponding entry in dependency-groups
+        # and add dependency information for existing groups
+        poetry_include_groups = {}
+        for group_name, group_config in tool_poetry_groups.items():
+            poetry_include_groups[group_name] = group_config.get("include-groups", [])
+            if package.has_dependency_group(group_name):
+                group = package.dependency_group(group_name)
+            else:
+                group = DependencyGroup(
+                    name=group_name,
+                    optional=group_config.get("optional", False),
+                )
+                package.add_dependency_group(group)
+            cls._add_package_poetry_group_dependencies(
+                package=package,
+                group=group,
+                dependencies=group_config.get("dependencies", {}),
+            )
+
+        for group_name, include_groups in chain(
+            pep739_include_groups.items(), poetry_include_groups.items()
+        ):
+            if include_groups:
+                current_group = package.dependency_group(group_name)
+                for name in include_groups:
+                    try:
+                        # `name` isn't normalized,
+                        # but `.dependency_group()` handles that.
+                        group_to_include = package.dependency_group(name)
+                    except ValueError as e:
+                        raise ValueError(
+                            f"Group '{group_name}' includes group '{name}'"
+                            " which is not defined."
+                        ) from e
+
+                    current_group.include_dependency_group(group_to_include)
+
     @classmethod
     def _prepare_formats(
         cls,
@@ -593,6 +736,14 @@ class Factory:
         ]
         result["errors"] += tool_poetry_validation_errors
 
+        dependency_groups = toml_data.get("dependency-groups")
+        if dependency_groups is not None:
+            dependency_groups_validation_errors = [
+                e.replace("data", "dependency-groups")
+                for e in validate_object(dependency_groups, "dependency-groups-schema")
+            ]
+            result["errors"] += dependency_groups_validation_errors
+
         # Check for required fields if package mode.
         # In non-package mode, there are no required fields.
         package_mode = tool_poetry.get("package-mode", True)
@@ -614,7 +765,13 @@ class Factory:
                 ' Use "poetry.group.dev.dependencies" instead.'
             )
 
+        cls._validate_dependency_groups(toml_data, result)
+
         if strict:
+            # Validate [project] section
+            if project:
+                cls._validate_project(project, result)
+
             # Validate relation between [project] and [tool.poetry]
             cls._validate_legacy_vs_project(toml_data, result)
 
@@ -622,6 +779,85 @@ class Factory:
 
         return result
 
+    @classmethod
+    def _validate_dependency_groups(
+        cls, toml_data: dict[str, Any], result: dict[str, list[str]]
+    ) -> None:
+        """Ensure that there are no duplicated dependency groups
+        and that they do not include themselves."""
+        original_names = defaultdict(set)
+        group_includes: dict[NormalizedName, list[NormalizedName]] = {}
+
+        for group_name, dependencies in toml_data.get("dependency-groups", {}).items():
+            normalized_group_name = canonicalize_name(group_name)
+            original_names[normalized_group_name].add(group_name)
+            for constraint in dependencies:
+                if isinstance(constraint, dict) and (
+                    include := constraint.get("include-group")
+                ):
+                    group_includes.setdefault(normalized_group_name, []).append(
+                        canonicalize_name(include)
+                    )
+
+        poetry_config = toml_data.get("tool", {}).get("poetry", {})
+        for group_name, group_config in poetry_config.get("group", {}).items():
+            normalized_group_name = canonicalize_name(group_name)
+            original_names[normalized_group_name].add(group_name)
+            if include_groups := group_config.get("include-groups", []):
+                group_includes[normalized_group_name] = [
+                    canonicalize_name(name) for name in include_groups
+                ]
+
+        for normed_name, names in original_names.items():
+            if len(names) > 1:
+                result["errors"].append(
+                    "Duplicate dependency group name after normalization:"
+                    f" {normed_name} ({', '.join(sorted(names))})"
+                )
+
+        for root in group_includes:
+            # group, path to group, ancestors
+            stack: list[
+                tuple[NormalizedName, list[NormalizedName], set[NormalizedName]]
+            ] = [(root, [], {root})]
+            while stack:
+                group, path, ancestors = stack.pop()
+                for include in group_includes.get(group, []):
+                    new_path = [*path, include]
+                    if include in ancestors:
+                        result["errors"].append(
+                            f"Cyclic dependency group include in {root}:"
+                            f" {' -> '.join(new_path)}"
+                        )
+                    else:
+                        stack.append((include, new_path, ancestors | {include}))
+
+    @classmethod
+    def _validate_project(
+        cls, project: dict[str, Any], result: dict[str, list[str]]
+    ) -> None:
+        if (project_license := project.get("license")) is not None:
+            if isinstance(project_license, str):
+                try:
+                    canonicalize_license_expression(project_license)
+                except InvalidLicenseExpression:
+                    result["warnings"].append(
+                        "[project.license] is not a valid SPDX expression."
+                        " This is deprecated and will raise an error in the future."
+                    )
+            else:
+                result["warnings"].append(
+                    "Defining [project.license] as a table is deprecated."
+                    " [project.license] should be a valid SPDX license expression."
+                    " License files can be referenced in [project.license-files]."
+                )
+
+        for classifier in project.get("classifiers", []):
+            if classifier.startswith("License :: "):
+                result["warnings"].append(
+                    "License classifiers are deprecated. Use [project.license] instead."
+                )
+
     @classmethod
     def _validate_legacy_vs_project(
         cls, toml_data: dict[str, Any], result: dict[str, list[str]]
@@ -731,7 +967,7 @@ class Factory:
             result["warnings"].append(
                 "[tool.poetry.dependencies] is set but [project.dependencies] is not"
                 " and 'dependencies' is not in [project.dynamic]."
-                " You should either migrate [tool.poetry.depencencies] to"
+                " You should either migrate [tool.poetry.dependencies] to"
                 " [project.dependencies] (if you do not need Poetry-specific features)"
                 " or add [project.dependencies] in addition to"
                 " [tool.poetry.dependencies] or add 'dependencies' to"
@@ -756,6 +992,12 @@ class Factory:
     def _validate_strict(
         cls, config: dict[str, Any], result: dict[str, list[str]]
     ) -> None:
+        for classifier in config.get("classifiers", []):
+            if classifier.startswith("License :: "):
+                result["warnings"].append(
+                    "License classifiers are deprecated. Use [project.license] instead."
+                )
+
         if "dependencies" in config:
             python_versions = config["dependencies"].get("python")
             if python_versions == "*":
