@@ -66,14 +66,14 @@ class Dependency(PackageSpecification):
 
         self._constraint: VersionConstraint
         self._pretty_constraint: str
-        self.constraint = constraint  # type: ignore[assignment]
+        self.constraint = constraint
 
         self._optional = optional
 
         if not groups:
             groups = [MAIN_GROUP]
 
-        self._groups = frozenset(groups)
+        self.groups = frozenset(canonicalize_name(g) for g in groups)
         self._allows_prereleases = allows_prereleases
         # "_develop" is only required for enriching [project] dependencies
         self._develop = False
@@ -115,10 +115,6 @@ class Dependency(PackageSpecification):
     def pretty_name(self) -> str:
         return self._pretty_name
 
-    @property
-    def groups(self) -> frozenset[str]:
-        return self._groups
-
     @property
     def python_versions(self) -> str:
         return self._python_versions
@@ -329,12 +325,20 @@ class Dependency(PackageSpecification):
 
     def with_constraint(self: T, constraint: str | VersionConstraint) -> T:
         dependency = self.clone()
-        dependency.constraint = constraint  # type: ignore[assignment]
+        dependency.constraint = constraint
+        return dependency
+
+    def with_groups(self, groups: Iterable[str]) -> Dependency:
+        dependency = self.clone()
+        dependency.groups = frozenset(canonicalize_name(g) for g in groups)
         return dependency
 
     @classmethod
     def create_from_pep_508(
-        cls, name: str, relative_to: Path | None = None
+        cls,
+        name: str,
+        relative_to: Path | None = None,
+        groups: Iterable[str] | None = None,
     ) -> Dependency:
         """
         Resolve a PEP-508 requirement string to a `Dependency` instance. If a
@@ -374,7 +378,7 @@ class Dependency(PackageSpecification):
             link = Link(req.url)
         else:
             path_str = os.path.normpath(os.path.abspath(name))  # noqa: PTH100
-            p, extras = strip_extras(path_str)
+            p, _extras = strip_extras(path_str)
             if cached_is_dir(p) and (os.path.sep in name or name.startswith(".")):
                 if not is_python_project(Path(name)):
                     raise ValueError(
@@ -416,10 +420,15 @@ class Dependency(PackageSpecification):
                     rev=url.rev,
                     directory=url.subdirectory,
                     extras=req.extras,
+                    groups=groups,
                 )
             elif link.scheme == "git":
                 dep = VCSDependency(
-                    name, "git", link.url_without_fragment, extras=req.extras
+                    name,
+                    "git",
+                    link.url_without_fragment,
+                    extras=req.extras,
+                    groups=groups,
                 )
             elif link.scheme in ("http", "https"):
                 dep = URLDependency(
@@ -427,6 +436,7 @@ class Dependency(PackageSpecification):
                     link.url_without_fragment,
                     directory=link.subdirectory_fragment,
                     extras=req.extras,
+                    groups=groups,
                 )
             elif is_file_uri:
                 # handle RFC 8089 references
@@ -437,6 +447,7 @@ class Dependency(PackageSpecification):
                     base=relative_to,
                     subdirectory=link.subdirectory_fragment,
                     extras=req.extras,
+                    groups=groups,
                 )
             else:
                 with suppress(ValueError):
@@ -446,17 +457,18 @@ class Dependency(PackageSpecification):
                         path=Path(req.url),
                         base=relative_to,
                         extras=req.extras,
+                        groups=groups,
                     )
 
             if dep is None:
-                dep = Dependency(name, version or "*", extras=req.extras)
+                dep = Dependency(name, version or "*", extras=req.extras, groups=groups)
 
             if version:
                 dep._constraint = parse_constraint(version)
         else:
             constraint: VersionConstraint | str
             constraint = req.constraint if req.pretty_constraint else "*"
-            dep = Dependency(name, constraint, extras=req.extras)
+            dep = Dependency(name, constraint, extras=req.extras, groups=groups)
 
         if req.marker:
             dep.marker = req.marker
@@ -499,12 +511,13 @@ def _make_file_or_dir_dep(
     base: Path | None = None,
     subdirectory: str | None = None,
     extras: Iterable[str] | None = None,
+    groups: Iterable[str] | None = None,
 ) -> FileDependency | DirectoryDependency | None:
     """
     Helper function to create a file or directoru dependency with the given arguments.
 
     If path is not a file or directory that exists, a guess is made based on the suffix
-    of the given path. This is done to prevent dependendencies from being parsed as normal
+    of the given path. This is done to prevent dependencies from being parsed as normal
     dependencies. This allows for downstream error handling.
 
     See also: poetry#10068
@@ -523,10 +536,10 @@ def _make_file_or_dir_dep(
 
     if is_file:
         return FileDependency(
-            name, path, base=base, directory=subdirectory, extras=extras
+            name, path, base=base, directory=subdirectory, extras=extras, groups=groups
         )
 
     if subdirectory:
         path = path / subdirectory
 
-    return DirectoryDependency(name, path, base=base, extras=extras)
+    return DirectoryDependency(name, path, base=base, extras=extras, groups=groups)
